# -*- coding: utf-8 -*-
"""
Created on Thu Mar  3 09:19:26 2022

@author: rusli001
"""

# import necessary package
import os
import flopy
import flopy.utils.binaryfile as bf
import flopy.discretization as fgrid
from flopy.utils.gridintersect import GridIntersect
import geopandas as gpd
import numpy as np
import math

# read and compile necessary geography data input
basin = gpd.read_file('shp/groundwater basin/domain_clip4.shp')
basin_geometry = basin.iloc[0].geometry
delr = 100
delc = 100
nrow = math.ceil((basin_geometry.bounds[3] - basin_geometry.bounds[1])/delc)
ncol = math.ceil((basin_geometry.bounds[2] - basin_geometry.bounds[0])/delr)
s_grid = fgrid.StructuredGrid(delc * np.ones(nrow), delr * np.ones(ncol), xoff = basin_geometry.bounds[0], yoff = basin_geometry.bounds[1])
ix = GridIntersect(s_grid, method = 'vertex', rtree = True)
obs = gpd.read_file('shp/observation point/data_layer1_2004/obs_lay1.shp')

# read MODFLOW run result
head_bf = bf.HeadFile('E:\PhD\\05 Analysis\\003 Paper 3\\006 20220221 - final model modified\\simulation results\\3 - full abstraction\\gwf_model.hds')
cbc_bf = bf.CellBudgetFile('E:\PhD\\05 Analysis\\003 Paper 3\\006 20220221 - final model modified\\simulation results\\3 - full abstraction\\gwf_model.cbc', precision = head_bf.precision)

# read specific discharge result
spdis = []
flowjaface = []
totim = np.arange(30, 5070, 30, dtype = 'int')
totim_len = len(totim)

i = 0
for i in range(totim_len):
    spdis.append(cbc_bf.get_data(totim = totim[i], text = "DATA-SPDIS"))
    i += 1

i = 0
for i in range(totim_len):
    flowjaface.append(cbc_bf.get_data(totim = totim[i], text = "FLOW-JA-FACE"))
    i += 1

flowjaface_average = np.mean(flowjaface, axis = 0)
flowjaface_average = flowjaface_average[0][0][0]

# create node array for identifier
node = [[] for i in range(totim_len)]

i = 0
j = 0
for i in range(totim_len):
    for j in range(int(len(spdis[0][0]))):
        node[i].append(spdis[i][0][j][0])
        j += 1
    i += 1

# add specific discharge result from all timestep per layer
qz_1 = [[] for i in range(totim_len)]
qz_sp_1 = np.empty(len(totim))

i = 0
j = 0
for i in range(totim_len):
    for j in range(int(len(spdis[0][0])/3)):
        qz_1[i].append(spdis[i][0][j][5])
        j += 1
    i += 1

i = 0
for i in range(totim_len):
    qz_sp_1[i] = np.sum(qz_1[i])
    i += 1

qz_2 = [[] for i in range(totim_len)]
qz_sp_2 = np.empty(len(totim))

i = 0
j = 0
for i in range(totim_len):
    for j in range(int(len(spdis[0][0])/3)):
        qz_2[i].append(spdis[i][0][j+171622][5])
        j += 1
    i += 1

i = 0
for i in range(totim_len):
    qz_sp_2[i] = np.sum(qz_2[i])
    i += 1

qz_3 = [[] for i in range(totim_len)]
qz_sp_3 = np.empty(len(totim))

i = 0
j = 0
for i in range(totim_len):
    for j in range(int(len(spdis[0][0])/3)):
        qz_3[i].append(spdis[i][0][j+171622+171622][5])
        j += 1
    i += 1

i = 0
for i in range(totim_len):
    qz_sp_3[i] = np.sum(qz_3[i])
    i += 1

np.savetxt('qz_lay1.csv', qz_sp_1, delimiter = ',')
np.savetxt('qz_lay2.csv', qz_sp_2, delimiter = ',')
np.savetxt('qz_lay3.csv', qz_sp_3, delimiter = ',')

# read RCH result
rch = []

i = 0
for i in range(totim_len):
    rch.append(cbc_bf.get_data(totim = totim[i], text = "RCH"))
    i += 1

rch_ = [[] for i in range(totim_len)]
rch_budget = np.empty(len(totim))

i = 0
j = 0
for i in range(totim_len):
    for j in range(int(len(spdis[0][0])/3)):
        rch_[i].append(rch[i][0][j][2])
        j += 1
    i += 1

i = 0
for i in range(totim_len):
    rch_budget[i] = np.sum(rch_[i])
    i += 1

np.savetxt('rch.csv', rch_budget, delimiter = ',')

# read WEL result
wel = []

i = 0
for i in range(totim_len):
    wel.append(cbc_bf.get_data(totim = totim[i], text = "WEL"))
    i += 1

wel_ = [[] for i in range(totim_len)]
wel_budget = np.empty(len(totim))

i = 0
j = 0
for i in range(totim_len):
    for j in range(int(len(wel[0][0]))):
        wel_[i].append(wel[i][0][j][2])
        j += 1
    i += 1

i = 0
for i in range(totim_len):
    wel_budget[i] = np.sum(wel_[i])
    i += 1

np.savetxt('wel.csv', wel_budget, delimiter = ',')

# read RIV result
riv = []

i = 0
for i in range(totim_len):
    riv.append(cbc_bf.get_data(totim = totim[i], text = "RIV"))
    i += 1

riv_ = [[] for i in range(totim_len)]
riv_budget = np.empty(len(totim))

i = 0
j = 0
for i in range(totim_len):
    for j in range(int(len(riv[0][0]))):
        riv_[i].append(riv[i][0][j][2])
        j += 1
    i += 1

i = 0
for i in range(totim_len):
    riv_budget[i] = np.sum(riv_[i])
    i += 1

np.savetxt('riv.csv', riv_budget, delimiter = ',')

# read STO-SY result
sto_sy = []

i = 0
for i in range(totim_len):
    sto_sy.append(cbc_bf.get_data(totim = totim[i], text = "STO-SY"))
    i += 1

sto_sy_budget = np.empty(len(totim))

i = 0
for i in range(totim_len):
    sto_sy_budget[i] = np.sum(sto_sy[i][0][0])
    i += 1

np.savetxt('sto_sy.csv', sto_sy_budget, delimiter = ',')

# read STO-SS result
sto_ss = []

i = 0
for i in range(totim_len):
    sto_ss.append(cbc_bf.get_data(totim = totim[i], text = "STO-SS"))
    i += 1

sto_ss_lay1 = np.empty(len(totim))
sto_ss_lay2 = np.empty(len(totim))
sto_ss_lay3 = np.empty(len(totim))

i = 0
for i in range(totim_len):
#    sto_ss_lay1[i] = np.sum(sto_ss[i][0][0])
    sto_ss_lay2[i] = np.sum(sto_ss[i][0][1])
#    sto_ss_lay3[i] = np.sum(sto_ss[i][0][2])
    i += 1

np.savetxt('sto_ss_lay1.csv', sto_ss_lay1, delimiter = ',')
np.savetxt('sto_ss_lay2.csv', sto_ss_lay2, delimiter = ',')
np.savetxt('sto_ss_lay3.csv', sto_ss_lay3, delimiter = ',')

########################################## plot test #####################################################

import matplotlib.pyplot as plt
import pandas as pd
from scipy.interpolate import griddata

sim_name = "mfsim.nam"
sim_path = os.path.join('E:\PhD\\05 Analysis\\003 Paper 3\\006 20220221 - final model modified\\simulation results\\3 - full abstraction')
sim = flopy.mf6.MFSimulation.load(sim_name=sim_name, sim_ws=sim_path)
gwf = sim.get_model("gwf_model")

head_0 = head_bf.get_data(totim = 30.0)
head_t = head_bf.get_data(totim = 5040.0)
drawdown = head_0 - head_t

basin_intersect = ix.intersects(basin_geometry)

idomain = np.zeros((4, nrow, ncol))
for i in basin_intersect.cellids:
    idomain[:, i[0], i[1]] = 1
for i in basin_intersect.cellids:
    idomain[3, i[0], i[1]] = 0

# define initial (bound to change) DIS_0 package
dis = flopy.mf6.ModflowGwfdis(gwf, length_units = "METERS", nlay = 4, ncol = ncol, nrow = nrow,
                              delr = delr, delc = delc, top = 100, botm = 0, idomain = idomain)

gwf.modelgrid.set_coord_info(xoff = basin_geometry.bounds[0], yoff = basin_geometry.bounds[1], angrot = 0, epsg = 32748)

# assign MERIT-DEM-based elevation
dem = pd.read_csv('csv/DIS - surface - MERIT-DEM-based elevation.csv')
points = dem[['Easting', 'Northing']].values
values = dem['Elevation'].values
grid_x = gwf.modelgrid.xcellcenters
grid_y = gwf.modelgrid.ycellcenters
dis_top = griddata(points, values, (grid_x, grid_y), method = 'nearest')

# save the result as csv
drawdown_csv = []

i = 0
j = 0
for i in range(nrow):
    for j in range(ncol):
        drawdown_csv.append([grid_x[0][j], grid_y[i][0], drawdown[0][i][j]])
        j += 1            
    i += 1

np.savetxt('drawdown.csv', drawdown, delimiter = ',')

# save the result as csv
idomain_csv = []

i = 0
j = 0
for i in range(nrow):
    for j in range(ncol):
        idomain_csv.append([grid_x[0][j], grid_y[i][0], idomain[0][i][j]])
        j += 1            
    i += 1

np.savetxt('idomain.csv', idomain_csv, delimiter = ',')

# plot the mapview check
fig, ax1 = plt.subplots(figsize = (15,10))
topview = flopy.plot.PlotMapView(model = gwf, ax = ax1)
linecollection = topview.plot_grid(linewidth = 0.1)
#surface_elevation = topview.plot_array(dis_top, alpha = 0.5)
#k = 0
#for k in range(len(abs_well)):
#    abstraction_plot = topview.plot_vector(vx = abs_well_geometry[k], vy = well[0][k][1])
#    k += 1
#cb_1 = fig.colorbar(surface_elevation, ax = ax1, shrink = 0.5, orientation = 'horizontal')
#cb_1.set_label('Surface elevation')
head_levels = np.linspace(np.nanmin(head_bf[0]), np.nanmax(head_bf[0]), num = 20)
head_plot = topview.contour_array(head_bf[0], levels = head_levels, cmap = 'plasma')
head_plot = topview.plot_array(head_bf[0], cmap = 'YlGnBu')
#cbaxes = inset_axes(ax1, width = '30%', height = '3%', loc = 4)
#cb_2 = fig.colorbar(head_plot, ax = ax1, format = '%.0f', cax = cbaxes, orientation = 'horizontal')
#cb_2.set_label('Simulated heads')
#cb_2.mappable.set_clim(600, 1600)
#cbaxes.xaxis.set_ticks_position('top')
#quiver = topview.plot_specific_discharge(spd, istep = 20, jstep = 20)
#riv_plot = river.plot(ax = ax1, edgecolor = 'red', linewidth = 1)
#bc = topview.plot_ibound()
#actives = topview.plot_inactive(color_noflow = 'white')
#basin.plot(ax = ax1, facecolor = 'none', edgecolor = 'black', linewidth = 2)
#ax1.set_xlim(760000, 830000)
#ax1.set_xticklabels([])
#ax1.set_ylim(9195000, 9250000)
#ax1.set_yticklabels([])
#fig.show()

# plot the cross-section check
#fig = plt.figure(figsize = (15,10))
#ax2 = fig.add_subplot(1, 1, 1)
#x_section = flopy.plot.PlotCrossSection(model = gwf, ax = ax2, line = {'Row': 155})
#lines = x_section.plot_grid(linewidth = 0.05)
#hk_plot = x_section.plot_array(np.log(flopy_npf_k))
#bc = x_section.plot_ibound(head = head)
#ic_plot = x_section.plot_array(flopy_ic_strt_plotxs)
#ic_plot_xs_colorbar = fig.colorbar(ic_plot)
#ic_line = x_section.plot_surface(flopy_ic_strt_plotxs[0], masked_values = [999.], color = 'blue', lw = 2)
#head_plot_xs = x_section.plot_array(head, head = head, alpha = 0.5, cmap = 'RdBu')
#head_line = x_section.plot_surface(head[0], masked_values = [999.], color = 'blue', lw = 2)
#head_plot_xs_colorbar = fig.colorbar(head_plot_xs)
#ax2.set_ylim(300, 2500)
#fig.show()